<?php
/**
 * Filters to alter the core plugin functionality
 *
 * @package   fonts-plugin-pro
 * @copyright Copyright (c) 2019, Danny Cooper
 * @license   http://opensource.org/licenses/gpl-2.0.php GNU Public License
 */

/**
 * Add hyperlink control.
 *
 * @param array $elements An array of elements from the olympus-google-fonts plugin.
 * @return array $elements An updated array of elements with hyperlinks.
 */
function fpp_add_hyperlinks( $elements ) {
	$elements['fpp_content_hyperlinks'] = array(
		'label'       => esc_html__( 'Hyperlinks', 'fonts-plugin-pro' ),
		'description' => esc_html__( 'Customize the typography and styling of links.', 'fonts-plugin-pro' ),
		'section'     => 'ogf_advanced__content',
		'priority'    => '15',
		'selectors'   => '.content a, .entry-content a, .post-content a, .page-content a, .post-excerpt a, .entry-summary a, .entry-excerpt a, .excerpt a, .elementor p a, .elementor-widget-theme-post-content a, .elementor-widget-text-editor a',
	);
	$elements['fpp_sidebar_hyperlinks'] = array(
		'label'       => esc_html__( 'Hyperlinks', 'fonts-plugin-pro' ),
		'description' => esc_html__( 'Customize the typography and styling of links.', 'fonts-plugin-pro' ),
		'section'     => 'ogf_advanced__sidebar',
		'priority'    => '15',
		'selectors'   => '.widget-area a, .widget a, .sidebar a, #sidebar a',
	);
	$elements['fpp_footer_hyperlinks']  = array(
		'label'       => esc_html__( 'Hyperlinks', 'fonts-plugin-pro' ),
		'description' => esc_html__( 'Customize the typography and styling of links.', 'fonts-plugin-pro' ),
		'section'     => 'ogf_advanced__footer',
		'priority'    => '15',
		'selectors'   => 'footer a, #footer a, .footer a, .site-footer a, .site-info a',
	);
	return $elements;
}
add_filter( 'ogf_elements', 'fpp_add_hyperlinks' );

/**
 * Helper function to build the CSS variables.
 *
 * @param array  $options The user set options.
 * @param string $device The device we are building CSS for (desktop|tablet|mobile).
 * @param string $breakpoint The device breakpoint (example: 600px).
 */
function fpp_responsive_styles( $options, $device, $breakpoint ) {
	$return = '@media only screen and (max-width: ' . esc_attr( $breakpoint ) . ') {' . PHP_EOL;

	foreach ( $options as $key => $value ) {
		$font_size   = get_theme_mod( $key . '_font_size_' . $device, false );
		$line_height = get_theme_mod( $key . '_line_height_' . $device, false );

		if ( $font_size || $line_height ) {
			$return .= $value['selectors'] . '{' . PHP_EOL;

			// Return font-size CSS.
			if ( $font_size ) {
				$return .= sprintf(
					'font-size: %s;' . PHP_EOL,
					floatval( $font_size ) . 'px' . ogf_is_forced()
				);
			}

			// Return font line-height CSS.
			if ( $line_height && '0' !== $line_height ) {
				$return .= sprintf(
					'line-height: %s;' . PHP_EOL,
					floatval( $line_height ) . ogf_is_forced()
				);
			}

			$return .= '}' . PHP_EOL;
		}
	}

	$return .= ' }' . PHP_EOL;

	echo wp_kses_post( $return );
}

/**
 * Output the mobile responsive CSS.
 */
function fpp_mobile_styles() {
	$mobile_size = apply_filters( 'fpp_mobile_breakpoint', '400px' );
	fpp_responsive_styles( ogf_get_elements(), 'mobile', $mobile_size );
	fpp_responsive_styles( ogf_get_custom_elements(), 'mobile', $mobile_size );
}
add_action( 'ogf_after_inline_styles', 'fpp_tablet_styles', 10 );

/**
 * Output the tablet responsive CSS.
 */
function fpp_tablet_styles() {
	$tablet_size = apply_filters( 'fpp_tablet_breakpoint', '800px' );
	fpp_responsive_styles( ogf_get_elements(), 'tablet', $tablet_size );
	fpp_responsive_styles( ogf_get_custom_elements(), 'tablet', $tablet_size );
}
add_action( 'ogf_after_inline_styles', 'fpp_mobile_styles', 20 );

/**
 * Add text decoration setting to controls not in the disallowed list.
 *
 * @param array  $array An array of disallowed controls.
 * @param string $id ID of current control.
 *
 * @return array $array An updated control array possibly including the text-decoration setting.
 */
function fpp_add_text_decoration_control( $array, $id ) {
	$disallowed_controls = [
		'ogf_body',
		'ogf_inputs',
		'ogf_post_page_content',
		'ogf_blockquotes',
		'ogf_sidebar_content',
		'ogf_footer_content',
	];

	if ( in_array( $id, $disallowed_controls ) ) {
		return $array;
	}

	$array['text_decoration'] = $id . '_text_decoration';
	return $array;
}
add_filter( 'ogf_typography_control_settings', 'fpp_add_text_decoration_control', 10, 2 );


/**
 * Remove the preconnect hint to fonts.gstatic.com.
 */
function fpp_remove_divi_preconnect() {
	remove_action( 'wp_enqueue_scripts', 'et_builder_preconnect_google_fonts', 9 );
}
add_action( 'init', 'fpp_remove_divi_preconnect' );


// Remove the aThemes resource hints.
remove_action( 'wp_head', 'sydney_preconnect_google_fonts' );
remove_action( 'wp_head', 'botiga_preconnect_google_fonts' );

/**
 * Filter the generated CSS file and remove subsets the website doesn't need.
 *
 * @param string $input The CSS file contents.
 *
 * @return string $output The CSS file contents with subsets removed.
 */
function fpp_remove_subsets_from_css( $input ) {
	$subsets_array = fpp_get_subsets_from_css( $input );

	if ( empty( $subsets_array ) ) {
		return $input;
	}

	$output = '';

	$disabled = get_theme_mod( 'fpp_disable_subsets', [] );

	foreach ( $subsets_array as $subset ) {
		$is_disabled = in_array( $subset['name'], $disabled );

		if ( $is_disabled && $subset['name'] !== 'latin' ) {
			continue;
		}

		$output .= '/* ' . $subset['name'] . ' */' . $subset['code'] . PHP_EOL;
	}

	return $output;
};
add_filter( 'fpp_local_css_output', 'fpp_remove_subsets_from_css', 10, 1 );

/**
 * Extract the subsets from a CSS file into an array.
 *
 * @param string $styles CSS file contents.
 *
 * @return array $matches Subsets extracted from CSS file.
 */
function fpp_get_subsets_from_css( $styles ) {
	$font_faces           = explode( '/* ', $styles );
	$extracted_font_faces = [];
	$matches              = [];

	foreach ( $font_faces as $ff ) {
		$extracted_font_faces[] = explode( ' */', $ff );
	}

	unset( $extracted_font_faces[0] );

	foreach ( $extracted_font_faces as $f ) {
		$matches[] = [
			'name' => $f[0],
			'code' => $f[1],
		];
	}

	return $matches;
}

/**
 * Filter the generated CSS file and add FPP attribution.
 *
 * @param string $input The CSS file contents.
 *
 * @return string $output The CSS file contents with attribution added.
 */
function fpp_add_data_to_css( $input ) {
	$output  = "/*\n";
	$output .= ' * ' . sprintf( 'Created by %s (v%s)', 'Fonts Plugin Pro', FPP_VERSION ) . "\n";
	$output .= ' * ' . date( 'F j, Y \a\t g:ia' ) . "\n";
	$output .= ' * https://fontsplugin.com' . "\n";
	$output .= "*/\n\n";
	$output .= $input;

	return $output;
};
add_filter( 'fpp_local_css_output', 'fpp_add_data_to_css', 20, 1 );
